/*
 *  identitycombo.cpp  -  combo box showing the up-to-date email identity list
 *  Program:  kalarm
 *  Copyright (C) 2004 by David Jarvie <software@astrojar.org.uk>
 *  This is a modified version of the KDEPIM file of the same name,
 *  Copyright (c) 2002 Marc Mutz <mutz@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <klocale.h>

#include "identity.h"
#include "identitymanager.h"
#include "identitycombo.moc"


namespace KPIM {


IdentityCombo::IdentityCombo(IdentityManager* manager, QWidget* parent, const char* name)
	: QComboBox(false, parent, name),
	  mIdentityManager(manager)
{
	reloadCombo();
	connect(this, SIGNAL(activated(int)), SLOT(slotEmitChanged(int)));
	connect(manager, SIGNAL(changed()), SLOT(slotIdentityManagerChanged()));
}

QString IdentityCombo::currentIdentityName() const
{
	int i = currentItem();
	return (i >= 0) ? mIdentities[i] : QString::null;
}

void IdentityCombo::setCurrentIdentity(const QString& name)
{
	int idx = mIdentities.findIndex(name);
	if (idx < 0  ||  idx == currentItem())
		return;

	blockSignals(true);    // just in case Qt gets fixed to emit activated() here
	setCurrentItem(idx);
	blockSignals(false);

	slotEmitChanged(idx);
}

void IdentityCombo::reloadCombo()
{
	clear();
	mIdentities = mIdentityManager->identities();
	if (!mIdentities.isEmpty())
	{
		QStringList identities = mIdentities;
		identities.first() = i18n("%1 (Default)").arg(identities.first());
		insertStringList(identities);
	}
}

void IdentityCombo::slotIdentityManagerChanged()
{
	QString oldIdentity = currentIdentityName();
	bool existed = !oldIdentity.isEmpty();
	blockSignals(true);
	reloadCombo();
	if (!mIdentities.isEmpty())
	{
		// Reselect the identity which was previously selected
		int idx = existed ? mIdentities.findIndex(oldIdentity) : -1;
		existed = existed && (idx < 0);
		setCurrentItem(idx < 0 ? 0 : idx);
	}
	blockSignals(false);
	if (existed)
		slotEmitChanged(currentItem());    // the old selection got deleted
}

void IdentityCombo::slotEmitChanged(int idx)
{
	emit identityChanged(mIdentities[idx]);
}

} // namespace KPIM
